import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:flutter/services.dart';
import 'package:moyoung_ble_plugin/impl/channel_names.dart';
import 'package:moyoung_ble_plugin/impl/moyoung_beans.dart';

import 'event_streams.dart';

class MoyoungBleImpl {
  static final MoyoungBleImpl _instance = MoyoungBleImpl();

  /// The default instance of [MoyoungBleImpl] to use.
  ///
  /// Defaults to [MoyoungBleImpl].
  static MoyoungBleImpl get instance => _instance;

  MethodChannel mBleScanMethodChannel =
      const MethodChannel(ChannelNames.methodScanDevice);
  MethodChannel mConnMethodChannel =
      const MethodChannel(ChannelNames.methodConnection);

  final _bleEveChannels = MYEventStreams();

  Future<bool> startScan(int scanPeriod) async {
    return await mBleScanMethodChannel
        .invokeMethod('startScan', <String, int>{"scanPeriod": scanPeriod});
  }

  Future<void> get cancelScan async {
    return await mBleScanMethodChannel.invokeMethod('cancelScan');
  }

  Future<bool> isConnected(String address) async {
    bool isConnected = await mConnMethodChannel
        .invokeMethod('isConnected', <String, String>{"address": address});
    return isConnected;
  }

  Future<void> connect(String address) async {
    return await mConnMethodChannel
        .invokeMethod('connect', <String, String>{"address": address});
  }

  Future<bool> disconnect() async {
    bool isSuccess = await mConnMethodChannel.invokeMethod('disconnect');
    return isSuccess;
  }

  Future<bool> closeGatt() async {
    bool isSuccess = await mConnMethodChannel.invokeMethod('closeGatt');
    return isSuccess;
  }

  Future<void> reconnect() async {
    return await mConnMethodChannel.invokeMethod('reconnect');
  }

  Future<void> connectDevice(ConnectDeviceBean connectDevice) async {
    // print(connectDevice);
    String jsonStr = connectDeviceBeanToJson(connectDevice);
    return await mConnMethodChannel.invokeMethod(
        'connectDevice', <String, String>{"connectDevice": jsonStr});
  }

  Future<void> get queryTime async {
    return await mConnMethodChannel.invokeMethod('queryTime');
  }

  Future<void> sendTimeSystem(int timeSystemType) async {
    return await mConnMethodChannel.invokeMethod(
        'sendTimeSystem', <String, int>{"timeSystemType": timeSystemType});
  }

  Future<int> get queryTimeSystem async {
    int time = await mConnMethodChannel.invokeMethod('queryTimeSystem');
    return time;
  }

  Future<String> get queryFirmwareVersion async {
    return await mConnMethodChannel.invokeMethod('queryFirmwareVersion');
  }

  Future<void> get queryDeviceBattery async {
    return await mConnMethodChannel.invokeMethod('queryDeviceBattery');
  }

  Future<void> get subscribeDeviceBattery async {
    return await mConnMethodChannel.invokeMethod('subscribeDeviceBattery');
  }

  Future<CheckFirmwareVersionBean> checkFirmwareVersion(
      FirmwareVersion info) async {
    String json = firmwareVersionToJson(info);
    String jsonStr = await mConnMethodChannel.invokeMethod(
        'checkFirmwareVersion', <String, String>{"firmwareVersion": json});
    return checkFirmwareVersionBeanFromJson(jsonStr);
  }

  Future<void> sendUserInfo(UserBean userInfo) async {
    String jsonStr = userBeanToJson(userInfo);
    return await mConnMethodChannel
        .invokeMethod('sendUserInfo', <String, String>{"userInfo": jsonStr});
  }

  Future<void> sendStepLength(int stepLength) async {
    return await mConnMethodChannel.invokeMethod(
        'sendStepLength', <String, int>{"stepLength": stepLength});
  }

  Future<void> sendTodayWeather(TodayWeatherBean todayWeatherInfo) async {
    String jsonStr = todayWeatherBeanToJson(todayWeatherInfo);
    return await mConnMethodChannel.invokeMethod(
        'sendTodayWeather', <String, String>{"todayWeatherInfo": jsonStr});
  }

  Future<void> sendFutureWeather(
      FutureWeatherListBean futureWeathersBean) async {
    String jsonStr = futureWeatherListBeanToJson(futureWeathersBean);
    return await mConnMethodChannel.invokeMethod(
        'sendFutureWeather', <String, String>{"futureWeatherInfo": jsonStr});
  }

  Stream<BleScanBean> get bleScanEveStm {
    return _bleEveChannels.bleScanEveStm;
  }

  Stream<dynamic> get connStateEveStm {
    return _bleEveChannels.stateEveStm;
  }

  Stream<StepsChangeBean> get stepsChangeEveStm {
    return _bleEveChannels.stepsChangeEveStm;
  }

  Stream<DeviceBatteryBean> get deviceBatteryEveStm {
    return _bleEveChannels.deviceBatteryEveStm;
  }

  Stream<WeatherChangeBean> get weatherChangeEveStm {
    return _bleEveChannels.weatherChangeEveStm;
  }

  Future<void> get querySteps async {
    await mConnMethodChannel.invokeMethod('querySteps');
  }

  Future<void> queryHistorySteps(int historyTimeType) async {
    await mConnMethodChannel.invokeMethod(
        'queryHistorySteps', <String, int>{"historyTimeType": historyTimeType});
  }

  Future<void> queryStepsDetail(int stepsDetailDateType) async {
    await mConnMethodChannel.invokeMethod('queryStepsDetail',
        <String, int>{"stepsDetailDateType": stepsDetailDateType});
  }

  Future<The24HourListBean> get get24HourCals async {
    String jsonStr = await mConnMethodChannel.invokeMethod('get24HourCals');
    return the24HourListBeanFromJson(jsonStr);
  }

  Future<The24HourListBean> get getAgo24HourCals async {
    String jsonStr = await mConnMethodChannel.invokeMethod('getAgo24HourCals');
    return the24HourListBeanFromJson(jsonStr);
  }

  Future<The24HourListBean> get get24HourDistances async {
    String jsonStr = await mConnMethodChannel.invokeMethod('get24HourDistances');
    return the24HourListBeanFromJson(jsonStr);
  }

  Future<The24HourListBean> get getAgo24HourDistances async {
    String jsonStr = await mConnMethodChannel.invokeMethod('getAgo24HourDistances');
    return the24HourListBeanFromJson(jsonStr);
  }

  Future<void> queryActionDetails(int actionDetailsType) async {
    await mConnMethodChannel.invokeMethod('queryActionDetails', <String, int>{"actionDetailsType": actionDetailsType});
  }

  Stream<StepsDetailBean> get stepsDetailEveStm {
    return _bleEveChannels.stepsDetailEveStm;
  }

  Stream<SleepBean> get sleepChangeEveStm {
    return _bleEveChannels.sleepChangeEveStm;
  }

  Future<void> get querySleep async {
    await mConnMethodChannel.invokeMethod('querySleep');
  }

  Future<void> get queryRemSleep async {
    await mConnMethodChannel.invokeMethod('queryRemSleep');
  }

  Future<void> sendGoalSleepTime(int minutes) async {
    await mConnMethodChannel.invokeMethod('sendGoalSleepTime', <String, int>{"goalTime": minutes});
  }

  Future<void> get queryGoalSleepTime async {
    await mConnMethodChannel.invokeMethod('queryGoalSleepTime');
  }

  Future<void> queryHistorySleep(int historyTimeType) async {
    await mConnMethodChannel.invokeMethod(
        'queryHistorySleep', <String, int>{"historyTimeType": historyTimeType});
  }

  Stream<OTABean> get oTAEveStm {
    return _bleEveChannels.oTAEveStm;
  }

  Future<void> startOTA(OtaBean otaBean) async {
    String jsonStr = otaBeanToJson(otaBean);
    await mConnMethodChannel
        .invokeMethod('startOTA', <String, String>{"otaBean": jsonStr});
  }

  Future<void> abortOTA(int type) async {
    await mConnMethodChannel
        .invokeMethod('abortOTA', <String, int>{"type": type});
  }

  Future<int> get queryDeviceOtaStatus async {
    int deviceOtaStatus =
        await mConnMethodChannel.invokeMethod('queryDeviceOtaStatus');
    return deviceOtaStatus;
  }

  Future<String> get queryHsOtaAddress async {
    String address = await mConnMethodChannel.invokeMethod('queryHsOtaAddress');
    return address;
  }

  Future<void> get enableHsOta async {
    await mConnMethodChannel.invokeMethod('enableHsOta');
  }

  Future<int> get queryOtaType async {
    int type = await mConnMethodChannel.invokeMethod('queryOtaType');
    return type;
  }

  Future<int> get queryUnitSystem async {
    int metricSystem = await mConnMethodChannel.invokeMethod('queryUnitSystem');
    return metricSystem;
  }

  Future<void> sendUnitSystem(int metricSystemType) async {
    await mConnMethodChannel.invokeMethod(
        'sendUnitSystem', <String, int>{"metricSystemType": metricSystemType});
  }

  Future<void> sendQuickView(bool quickViewState) async {
    await mConnMethodChannel.invokeMethod(
        'sendQuickView', <String, bool>{"quickViewState": quickViewState});
  }

  Future<bool> get queryQuickView async {
    bool quickViewState =
        await mConnMethodChannel.invokeMethod('queryQuickView');
    return quickViewState;
  }

  Future<void> sendQuickViewTime(PeriodTimeBean periodTimeInfo) async {
    String jsonStr = periodTimeBeanToJson(periodTimeInfo);
    await mConnMethodChannel.invokeMethod(
        'sendQuickViewTime', <String, String>{"periodTimeInfo": jsonStr});
  }

  Future<PeriodTimeResultBean> get queryQuickViewTime async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('queryQuickViewTime');
    PeriodTimeResultBean quickViewTime = periodTimeResultBeanFromJson(jsonStr);
    return quickViewTime;
  }

  Future<void> sendGoalSteps(int goalSteps) async {
    await mConnMethodChannel
        .invokeMethod('sendGoalSteps', <String, int>{"goalSteps": goalSteps});
  }

  Future<int> get queryGoalSteps async {
    int goalSteps = await mConnMethodChannel.invokeMethod('queryGoalSteps');
    return goalSteps;
  }

  Future<void> sendDailyGoals(DailyGoalsInfoBean dailyGoals) async {
    String josnStr = dailyGoalsInfoBeanToJson(dailyGoals);
    await mConnMethodChannel.invokeMethod(
        'sendDailyGoals', <String, dynamic>{"dailyGoals": josnStr});
  }

  Future<DailyGoalsInfoBean> get queryDailyGoals async {
    String jsonStr = await mConnMethodChannel.invokeMethod('queryDailyGoals');
    DailyGoalsInfoBean goalsInfoBean = dailyGoalsInfoBeanFromJson(jsonStr);
    return goalsInfoBean;
  }

  Future<void> sendTrainingDayGoals(DailyGoalsInfoBean dailyGoals) async {
    String josnStr = dailyGoalsInfoBeanToJson(dailyGoals);
    await mConnMethodChannel.invokeMethod(
        'sendTrainingDayGoals', <String, dynamic>{"dailyGoals": josnStr});
  }

  Future<DailyGoalsInfoBean> get queryTrainingDayGoals async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('queryTrainingDayGoals');
    DailyGoalsInfoBean goalsInfoBean = dailyGoalsInfoBeanFromJson(jsonStr);
    return goalsInfoBean;
  }

  Future<void> sendTrainingDays(TrainingDayInfoBean trainingDay) async {
    String jsonStr = trainingDayInfoBeanToJson(trainingDay);
    await mConnMethodChannel.invokeMethod(
        'sendTrainingDays', <String, dynamic>{"trainingDay": jsonStr});
  }

  Future<TrainingDayInfoBean> get queryTrainingDay async {
    String jsonStr = await mConnMethodChannel.invokeMethod('queryTrainingDay');
    TrainingDayInfoBean trainingDay = trainingDayInfoBeanFromJson(jsonStr);
    return trainingDay;
  }

  Future<void> sendDisplayWatchFace(int watchFaceType) async {
    await mConnMethodChannel.invokeMethod(
        'sendDisplayWatchFace', <String, int>{"watchFaceType": watchFaceType});
  }

  Future<int> get queryDisplayWatchFace async {
    int watchFaceType =
        await mConnMethodChannel.invokeMethod('queryDisplayWatchFace');
    return watchFaceType;
  }

  Future<WatchFaceLayoutBean> get queryWatchFaceLayout async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('queryWatchFaceLayout');
    WatchFaceLayoutBean watchFaceLayoutInfo =
        watchFaceLayoutBeanFromJson(jsonStr);
    return watchFaceLayoutInfo;
  }

  Future<void> sendWatchFaceLayout(
      WatchFaceLayoutBean watchFaceLayoutInfo) async {
    String jsonStr = watchFaceLayoutBeanToJson(watchFaceLayoutInfo);
    await mConnMethodChannel.invokeMethod('sendWatchFaceLayout',
        <String, String>{"watchFaceLayoutInfo": jsonStr});
  }

  Future<void> sendWatchFaceBackground(
      WatchFaceBackgroundBean watchFaceBackgroundInfo) async {
    String jsonStr = watchFaceBackgroundBeanToJson(watchFaceBackgroundInfo);
    await mConnMethodChannel.invokeMethod('sendWatchFaceBackground',
        <String, String>{"watchFaceBackgroundInfo": jsonStr});
  }

  Future<void> get abortWatchFaceBackground async {
    await mConnMethodChannel.invokeMethod('abortWatchFaceBackground');
  }

  Stream<FileTransBean> get fileTransEveStm {
    return _bleEveChannels.fileTransEveStm;
  }

  Future<SupportWatchFaceBean> get querySupportWatchFace async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('querySupportWatchFace');
    SupportWatchFaceBean supportWatchFaceInfo =
        supportWatchFaceBeanFromJson(jsonStr);
    return supportWatchFaceInfo;
  }

  Future<List<WatchFaceBean>> queryWatchFaceStore(
      WatchFaceStoreBean watchFaceStoreBean) async {
    String jsonStr = watchFaceStoreBeanToJson(watchFaceStoreBean);
    String str = await mConnMethodChannel.invokeMethod(
        'queryWatchFaceStore', <String, String>{"watchFaceStoreBean": jsonStr});
    WatchFaceStoreListBean watchFaceStore = watchFaceStoreListBeanFromJson(str);
    WatchFaceStore watchFaceInfo = watchFaceStore.watchFaceStoreInfo;
    return watchFaceInfo.list;
  }

  Future<WatchFaceIdBean> queryWatchFaceOfID(int id) async {
    String jsonStr = await mConnMethodChannel
        .invokeMethod('queryWatchFaceOfID', <String, int>{"id": id});
    WatchFaceIdBean? watchFaceId = watchFaceIdBeanFromJson(jsonStr);
    return watchFaceId;
  }

  Future<void> sendWatchFace(SendWatchFaceBean sendWatchFaceBean) async {
    String jsonStr = sendWatchFaceBeanToJson(sendWatchFaceBean);
    await mConnMethodChannel.invokeMethod(
        'sendWatchFace', <String, String>{"sendWatchFaceBean": jsonStr});
  }

  Future<void> get abortWatchFace {
    return mConnMethodChannel.invokeMethod('abortWatchFace');
  }

  Future<void> deleteWatchFace(int id) {
    return mConnMethodChannel
        .invokeMethod('deleteWatchFace', <String, int>{"id": id});
  }

  Future<int> get queryAvailableStorage async {
    return await mConnMethodChannel.invokeMethod("queryAvailableStorage");
  }

  Stream<FileTransBean> get wfFileTransEveStm {
    return _bleEveChannels.wfFileTransEveStm;
  }

  Future<void> sendAlarm(AlarmClockBean alarmInfo) async {
    String jsonStr = alarmClockBeanToJson(alarmInfo);
    await mConnMethodChannel
        .invokeMethod('sendAlarm', <String, String>{"alarmClockInfo": jsonStr});
  }

  Future<List<AlarmClockBean>> get queryAllAlarm async {
    String jsonStr = await mConnMethodChannel.invokeMethod('queryAllAlarm');
    var listDynamic = jsonDecode(jsonStr);
    List<AlarmClockBean> listStr = (listDynamic as List<dynamic>)
        .map((e) => AlarmClockBean.fromJson((e as Map<String, dynamic>)))
        .toList();

    return listStr;
  }

  Future<void> sendNewAlarm(AlarmClockBean alarmInfo) async {
    String jsonStr = alarmClockBeanToJson(alarmInfo);
    await mConnMethodChannel.invokeMethod(
        "sendNewAlarm", <String, String>{"alarmClockInfo": jsonStr});
  }

  Future<void> deleteNewAlarm(int id) async {
    await mConnMethodChannel
        .invokeMethod("deleteNewAlarm", <String, int>{"id": id});
  }

  Future<void> deleteAllNewAlarm() async {
    await mConnMethodChannel.invokeMethod("deleteAllNewAlarm");
  }

  Future<List<AlarmClockBean>> get queryAllNewAlarm async {
    String jsonStr = await mConnMethodChannel.invokeMethod("queryAllNewAlarm");
    var listStr = jsonDecode(jsonStr);
    List<AlarmClockBean> list = (listStr as List<dynamic>)
        .map((e) => AlarmClockBean.fromJson((e as Map<String, dynamic>)))
        .toList();
    return list;
  }

  Stream<HeartRateBean> get heartRateEveStm {
    return _bleEveChannels.heartRateEveStm;
  }

  Future<void> queryLastDynamicRate(String type) async {
    await mConnMethodChannel
        .invokeMethod('queryLastDynamicRate', <String, String>{"type": type});
  }

  Future<void> enableTimingMeasureHeartRate(int interval) async {
    await mConnMethodChannel.invokeMethod(
        'enableTimingMeasureHeartRate', <String, int>{"interval": interval});
  }

  Future<void> get disableTimingMeasureHeartRate async {
    await mConnMethodChannel.invokeMethod('disableTimingMeasureHeartRate');
  }

  Future<int> get queryTimingMeasureHeartRate async {
    int timeHR =
        await mConnMethodChannel.invokeMethod('queryTimingMeasureHeartRate');
    return timeHR;
  }

  Future<void> queryTodayHeartRate(int heartRateType) async {
    await mConnMethodChannel.invokeMethod(
        'queryTodayHeartRate', <String, int>{"heartRateType": heartRateType});
  }

  Future<void> queryPastHeartRate(int historyDay) async {
    await mConnMethodChannel.invokeMethod(
        'queryPastHeartRate', <String, int>{"historyDay": historyDay});
  }

  Future<void> get queryTrainingHeartRate async {
    await mConnMethodChannel.invokeMethod('queryTrainingHeartRate');
  }

  Future<void> get startMeasureOnceHeartRate async {
    await mConnMethodChannel.invokeMethod('startMeasureOnceHeartRate');
  }

  Future<void> get stopMeasureOnceHeartRate async {
    await mConnMethodChannel.invokeMethod('stopMeasureOnceHeartRate');
  }

  Future<void> get queryHistoryHeartRate async {
    await mConnMethodChannel.invokeMethod('queryHistoryHeartRate');
  }

  Stream<BloodPressureBean> get bloodPressureEveStm {
    return _bleEveChannels.bloodPressureEveStm;
  }

  Future<void> get startMeasureBloodPressure async {
    await mConnMethodChannel.invokeMethod('startMeasureBloodPressure');
  }

  Future<void> get stopMeasureBloodPressure async {
    await mConnMethodChannel.invokeMethod('stopMeasureBloodPressure');
  }

  Future<void> get enableContinueBloodPressure async {
    await mConnMethodChannel.invokeMethod('enableContinueBloodPressure');
  }

  Future<void> get disableContinueBloodPressure async {
    await mConnMethodChannel.invokeMethod('disableContinueBloodPressure');
  }

  Future<void> get queryContinueBloodPressureState async {
    await mConnMethodChannel.invokeMethod('queryContinueBloodPressureState');
  }

  Future<void> get queryLast24HourBloodPressure async {
    await mConnMethodChannel.invokeMethod('queryLast24HourBloodPressure');
  }

  Future<void> get queryHistoryBloodPressure async {
    await mConnMethodChannel.invokeMethod('queryHistoryBloodPressure');
  }

  Stream<BloodOxygenBean> get bloodOxygenEveStm {
    return _bleEveChannels.bloodOxygenEveStm;
  }

  Future<void> get startMeasureBloodOxygen async {
    await mConnMethodChannel.invokeMethod('startMeasureBloodOxygen');
  }

  Future<void> get stopMeasureBloodOxygen async {
    await mConnMethodChannel.invokeMethod('stopMeasureBloodOxygen');
  }

  Future<void> enableTimingMeasureBloodOxygen(int interval) async {
    await mConnMethodChannel.invokeMethod(
        'enableTimingMeasureBloodOxygen', <String, int>{"interval": interval});
  }

  Future<void> get disableTimingMeasureBloodOxygen async {
    await mConnMethodChannel.invokeMethod('disableTimingMeasureBloodOxygen');
  }

  Future<void> get queryTimingBloodOxygenMeasureState async {
    await mConnMethodChannel.invokeMethod('queryTimingBloodOxygenMeasureState');
  }

  Future<void> queryTimingBloodOxygen(String bloodOxygenTimeType) async {
    await mConnMethodChannel.invokeMethod('queryTimingBloodOxygen',
        <String, String>{"bloodOxygenTimeType": bloodOxygenTimeType});
  }

  Future<void> get enableContinueBloodOxygen async {
    await mConnMethodChannel.invokeMethod('enableContinueBloodOxygen');
  }

  Future<void> get disableContinueBloodOxygen async {
    await mConnMethodChannel.invokeMethod('disableContinueBloodOxygen');
  }

  Future<void> get queryContinueBloodOxygenState async {
    await mConnMethodChannel.invokeMethod('queryContinueBloodOxygenState');
  }

  Future<void> get queryLast24HourBloodOxygen async {
    await mConnMethodChannel.invokeMethod('queryLast24HourBloodOxygen');
  }

  Future<void> get queryHistoryBloodOxygen async {
    await mConnMethodChannel.invokeMethod('queryHistoryBloodOxygen');
  }

  Future<void> get enterCameraView async {
    await mConnMethodChannel.invokeMethod('enterCameraView');
  }

  Future<void> sendDelayTaking(int delayTime) async {
    await mConnMethodChannel.invokeMethod('sendDelayTaking', <String, int>{"delayTime": delayTime});
  }

  // Future<void> get queryDelayTaking async {
  //   await mConnMethodChannel.invokeMethod('queryDelayTaking');
  // }

  Future<void> get exitCameraView async {
    await mConnMethodChannel.invokeMethod('exitCameraView');
  }

  Stream<CameraBean> get cameraEveStm {
    return _bleEveChannels.cameraEveStm;
  }

  Stream<int> get phoneEveStm {
    return _bleEveChannels.phoneEveStm;
  }

  Stream<int> get deviceRssiEveStm {
    return _bleEveChannels.deviceRssiEveStm;
  }

  Future<void> get readDeviceRssi async {
    await mConnMethodChannel.invokeMethod('readDeviceRssi');
  }

  Future<void> setECGChangeListener(String ecgMeasureType) async {
    await mConnMethodChannel.invokeMethod('setECGChangeListener',
        <String, String>{"ecgMeasureType": ecgMeasureType});
  }

  Stream<EcgBean> get ecgEveStm {
    return _bleEveChannels.ecgEveStm;
  }

  Future<void> get startECGMeasure async {
    await mConnMethodChannel.invokeMethod('startECGMeasure');
  }

  Future<void> get stopECGMeasure async {
    await mConnMethodChannel.invokeMethod('stopECGMeasure');
  }

  Future<bool> get isNewECGMeasurementVersion async {
    bool newMeasurementVersion =
        await mConnMethodChannel.invokeMethod('isNewECGMeasurementVersion');
    return newMeasurementVersion;
  }

  Future<void> get queryLastMeasureECGData async {
    await mConnMethodChannel.invokeMethod('queryLastMeasureECGData');
  }

  Future<void> sendECGHeartRate(int heartRate) async {
    await mConnMethodChannel.invokeMethod(
        'sendECGHeartRate', <String, int>{"heartRate": heartRate});
  }

  Stream<FileTransBean> get contactAvatarEveStm {
    return _bleEveChannels.tactAvatarEveStm;
  }

  Future<void> sendDeviceLanguage(int language) async {
    await mConnMethodChannel.invokeMethod(
        'sendDeviceLanguage', <String, int>{"language": language});
  }

  Future<int> get queryDeviceVersion async {
    return await mConnMethodChannel.invokeMethod("queryDeviceVersion");
  }

  Future<DeviceLanguageBean> get queryDeviceLanguage async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('queryDeviceLanguage');
    DeviceLanguageBean deviceLanguage = deviceLanguageBeanFromJson(jsonStr);
    return deviceLanguage;
  }

  Future<void> sendOtherMessageState(bool state) async {
    await mConnMethodChannel
        .invokeMethod("sendOtherMessageState", <String, bool>{"state": state});
  }

  Future<bool> get queryOtherMessageState async {
    return await mConnMethodChannel.invokeMethod("queryOtherMessageState");
  }

  Future<void> sendMessage(MessageBean messageInfo) async {
    String jsonStr = messageBeanToJson(messageInfo);
    await mConnMethodChannel
        .invokeMethod('sendMessage', <String, String>{"messageInfo": jsonStr});
  }

  Future<void> get endCall async {
    await mConnMethodChannel.invokeMethod('endCall');
  }

  Future<void> sendCallContactName(String name) async {
    await mConnMethodChannel
        .invokeMethod("sendCallContactName", <String, String>{"name": name});
  }

  Future<List<int>> get queryMessageList async {
    String jsonStr = await mConnMethodChannel.invokeMethod("queryMessageList");
    List<int> list = jsonStr.split(",").map((e) => int.parse(e)).toList();
    return list;
  }

  Stream<String> get callNumberEveStm {
    return _bleEveChannels.callNumberEveStm;
  }

  Future<void> setNotification(List<int> ints) async {
    await mConnMethodChannel
        .invokeMethod('setNotification', <String, List<int>>{"ints": ints});
  }

  Future<List<int>> get getNotification async {
    String str = await mConnMethodChannel.invokeMethod('getNotification');
    List<int> list = str.split(",").map((e) => int.parse(e)).toList();
    return list;
  }

  Future<void> sendNotificationState(List<int> list) async {
    await mConnMethodChannel.invokeMethod('sendNotificationState', <String, List<int>>{"list": list});
  }

  Future<List<int>> get queryNotificationState async {
    String str = await mConnMethodChannel.invokeMethod('queryNotificationState');
    List<int> list = str.split(",").map((e) => int.parse(e)).toList();
    return list;
  }

  Future<void> sendSedentaryReminder(bool sedentaryReminder) async {
    await mConnMethodChannel.invokeMethod('sendSedentaryReminder',
        <String, bool>{"sedentaryReminder": sedentaryReminder});
  }

  Future<bool> get querySedentaryReminder async {
    bool sedentaryReminder =
        await mConnMethodChannel.invokeMethod('querySedentaryReminder');
    return sedentaryReminder;
  }

  Future<void> sendSedentaryReminderPeriod(
      SedentaryReminderPeriodBean sedentaryReminderPeriodInfo) async {
    String jsonStr =
        sedentaryReminderPeriodBeanToJson(sedentaryReminderPeriodInfo);
    await mConnMethodChannel.invokeMethod('sendSedentaryReminderPeriod',
        <String, String>{"sedentaryReminderPeriodInfo": jsonStr});
  }

  Future<SedentaryReminderPeriodBean> get querySedentaryReminderPeriod async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('querySedentaryReminderPeriod');
    SedentaryReminderPeriodBean sedentaryReminderPeriodInfo =
        sedentaryReminderPeriodBeanFromJson(jsonStr);
    return sedentaryReminderPeriodInfo;
  }

  Future<void> get findDevice async {
    await mConnMethodChannel.invokeMethod('findDevice');
  }

  Future<void> get shutDown async {
    await mConnMethodChannel.invokeMethod('shutDown');
  }

  Future<void> get reset async {
    await mConnMethodChannel.invokeMethod("reset");
  }

  Future<PeriodTimeResultBean> get queryDoNotDisturbTime async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('queryDoNotDisturbTime');
    PeriodTimeResultBean doNotDisturbTime =
        periodTimeResultBeanFromJson(jsonStr);
    return doNotDisturbTime;
  }

  Future<void> sendDoNotDisturbTime(PeriodTimeBean periodTimeInfo) async {
    String jsonStr = periodTimeBeanToJson(periodTimeInfo);
    await mConnMethodChannel.invokeMethod(
        'sendDoNotDisturbTime', <String, String>{"periodTimeInfo": jsonStr});
  }

  Future<void> sendBreathingLight(bool breathingLight) async {
    await mConnMethodChannel.invokeMethod(
        'sendBreathingLight', <String, bool>{"breathingLight": breathingLight});
  }

  Future<bool> get queryBreathingLight async {
    bool breathingLight =
        await mConnMethodChannel.invokeMethod('queryBreathingLight');
    return breathingLight;
  }

  Future<void> sendMenstrualCycle(MenstrualCycleBean info) async {
    String jsonStr = menstrualCycleBeanToJson(info);
    await mConnMethodChannel.invokeMethod(
        "sendMenstrualCycle", <String, String>{"menstrualCycleBean": jsonStr});
  }

  Future<MenstrualCycleBean> get queryMenstrualCycle async {
    String resultJson =
        await mConnMethodChannel.invokeMethod("queryMenstrualCycle");
    MenstrualCycleBean physiologcalPeriodBean =
        menstrualCycleBeanFromJson(resultJson);
    return physiologcalPeriodBean;
  }

  Future<void> get startFindPhone async {
    await mConnMethodChannel.invokeMethod("startFindPhone");
  }

  Future<void> get stopFindPhone async {
    await mConnMethodChannel.invokeMethod("stopFindPhone");
  }

  Stream<FindPhoneBean> get findPhoneEveStm {
    return _bleEveChannels.findPhoneEveStm;
  }

  Future<void> setPlayerState(int musicPlayer) async {
    await mConnMethodChannel.invokeMethod(
        "setPlayerState", <String, int>{"musicPlayerStateType": musicPlayer});
  }

  Future<void> sendSongTitle(String title) async {
    await mConnMethodChannel
        .invokeMethod("sendSongTitle", <String, String>{"title": title});
  }

  Future<void> sendLyrics(String lyrics) async {
    await mConnMethodChannel
        .invokeMethod("sendLyrics", <String, String>{"lyrics": lyrics});
  }

  Future<void> get closePlayerControl async {
    await mConnMethodChannel.invokeMethod("closePlayerControl");
  }

  Future<void> sendCurrentVolume(int volume) async {
    await mConnMethodChannel
        .invokeMethod("sendCurrentVolume", <String, int>{"volume": volume});
  }

  Future<void> sendMaxVolume(int volume) async {
    await mConnMethodChannel
        .invokeMethod("sendMaxVolume", <String, int>{"volume": volume});
  }

  Future<void> enableDrinkWaterReminder(
      DrinkWaterPeriodBean drinkWaterPeriodInfo) async {
    String jsonStr = drinkWaterPeriodBeanToJson(drinkWaterPeriodInfo);
    await mConnMethodChannel.invokeMethod('enableDrinkWaterReminder',
        <String, String>{"drinkWaterPeriodInfo": jsonStr});
  }

  Future<void> get disableDrinkWaterReminder async {
    await mConnMethodChannel.invokeMethod("disableDrinkWaterReminder");
  }

  Future<DrinkWaterPeriodBean> get queryDrinkWaterReminderPeriod async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod("queryDrinkWaterReminderPeriod");
    return drinkWaterPeriodBeanFromJson(jsonStr);
  }

  Future<void> setMaxHeartRate(MaxHeartRateBean maxHeartRateBean) async {
    String jsonStr = maxHeartRateBeanToJson(maxHeartRateBean);
    await mConnMethodChannel.invokeMethod(
        "setMaxHeartRate", <String, String>{"maxHeartRateBean": jsonStr});
  }

  Future<MaxHeartRateBean> get queryMaxHeartRate async {
    String jsonStr = await mConnMethodChannel.invokeMethod("queryMaxHeartRate");
    MaxHeartRateBean maxHeartRate = maxHeartRateBeanFromJson(jsonStr);
    return maxHeartRate;
  }

  Future<void> startTraining(int type) async {
    await mConnMethodChannel
        .invokeMethod("startTraining", <String, int>{"type": type});
  }

  Future<void> setTrainingState(int training) async {
    await mConnMethodChannel
        .invokeMethod("setTrainingState", <String, int>{"training": training});
  }

  Stream<int> get trainingStateEveStm {
    return _bleEveChannels.trainingStateEveStm;
  }

  Future<String> get getProtocolVersion async {
    String protocolVersion =
        await mConnMethodChannel.invokeMethod("getProtocolVersion");
    return protocolVersion;
  }

  Future<void> get startMeasureTemp async {
    await mConnMethodChannel.invokeMethod("startMeasureTemp");
  }

  Future<void> get stopMeasureTemp async {
    await mConnMethodChannel.invokeMethod("stopMeasureTemp");
  }

  Future<void> get enableTimingMeasureTemp async {
    await mConnMethodChannel.invokeMethod("enableTimingMeasureTemp");
  }

  Future<void> get disableTimingMeasureTemp async {
    return await mConnMethodChannel.invokeMethod("disableTimingMeasureTemp");
  }

  Future<String> get queryTimingMeasureTempState async {
    String tempState =
        await mConnMethodChannel.invokeMethod("queryTimingMeasureTempState");
    return tempState;
  }

  Future<void> queryTimingMeasureTemp(String tempTimeType) async {
    await mConnMethodChannel.invokeMethod("queryTimingMeasureTemp",
        <String, String>{"tempTimeType": tempTimeType});
  }

  Future<void> get sendGsensorCalibration async {
    await mConnMethodChannel.invokeMethod("sendGsensorCalibration");
  }

  Future<void> get enableContinueTemp async {
    await mConnMethodChannel.invokeMethod("enableContinueTemp");
  }

  Future<void> get disableContinueTemp async {
    await mConnMethodChannel.invokeMethod("disableContinueTemp");
  }

  Future<void> get queryContinueTempState async {
    await mConnMethodChannel.invokeMethod("queryContinueTempState");
  }

  Future<void> get queryLast24HourTemp async {
    await mConnMethodChannel.invokeMethod("queryLast24HourTemp");
  }

  Stream<TempChangeBean> get tempChangeEveStm {
    return _bleEveChannels.tempChangeEveStm;
  }

  Future<void> sendDisplayTime(int time) async {
    await mConnMethodChannel
        .invokeMethod("sendDisplayTime", <String, int>{"time": time});
  }

  Future<int> get queryDisplayTime async {
    int time = await mConnMethodChannel.invokeMethod("queryDisplayTime");
    return time;
  }

  Future<void> enableHandWashingReminder(
      HandWashingPeriodBean handWashingPeriodInfo) async {
    String jsonStr = handWashingPeriodBeanToJson(handWashingPeriodInfo);
    await mConnMethodChannel.invokeMethod("enableHandWashingReminder",
        <String, String>{"handWashingPeriodInfo": jsonStr});
  }

  Future<void> get disableHandWashingReminder async {
    await mConnMethodChannel.invokeMethod("disableHandWashingReminder");
  }

  Future<HandWashingPeriodBean> get queryHandWashingReminderPeriod async {
    String period =
        await mConnMethodChannel.invokeMethod("queryHandWashingReminderPeriod");
    return handWashingPeriodBeanFromJson(period);
  }

  Future<void> sendLocalCity(String city) async {
    await mConnMethodChannel
        .invokeMethod("sendLocalCity", <String, String>{"city": city});
  }

  Future<void> sendTempUnit(int temp) async {
    await mConnMethodChannel
        .invokeMethod("sendTempUnit", <String, int>{"temp": temp});
  }

  Future<void> get queryTempUnit async {
    await mConnMethodChannel.invokeMethod("queryTempUnit");
  }

  Future<void> sendBrightness(int brightness) async {
    await mConnMethodChannel.invokeMethod(
        "sendBrightness", <String, int>{"brightness": brightness});
  }

  Future<BrightnessBean> get queryBrightness async {
    String jsonStr = await mConnMethodChannel.invokeMethod("queryBrightness");
    BrightnessBean brightness = brightnessBeanFromJson(jsonStr);
    return brightness;
  }

  Future<String> get queryBtAddress async {
    String address = await mConnMethodChannel.invokeMethod("queryBtAddress");
    return address;
  }

  Future<ContactConfigBean> get checkSupportQuickContact async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod("checkSupportQuickContact");
    ContactConfigBean contactConfigBean = contactConfigBeanFromJson(jsonStr);
    return contactConfigBean;
  }

  Future<int> get queryContactCount async {
    int count = await mConnMethodChannel.invokeMethod("queryContactCount");
    return count;
  }

  Stream<ContactListenBean> get contactEveStm {
    return _bleEveChannels.tactEveStm;
  }

  Future<bool> get queryContactNumberSymbol async {
    return await mConnMethodChannel.invokeMethod("queryContactNumberSymbol");
  }

  Future<void> sendContact(ContactBean info) async {
    String jsonStr = contactBeanToJson(info);
    await mConnMethodChannel
        .invokeMethod("sendContact", <String, String>{"contactBean": jsonStr});
  }

  Future<void> sendContactAvatar(ContactBean info) async {
    String jsonStr = contactBeanToJson(info);
    await mConnMethodChannel.invokeMethod(
        "sendContactAvatar", <String, String>{"contactAvatarBean": jsonStr});
  }

  Future<void> deleteContact(int index) async {
    await mConnMethodChannel
        .invokeMethod("deleteContact", <String, int>{"index": index});
  }

  Future<void> deleteContactAvatar(int index) async {
    await mConnMethodChannel
        .invokeMethod("deleteContactAvatar", <String, int>{"index": index});
  }

  Future<void> clearContact() async {
    await mConnMethodChannel.invokeMethod("clearContact");
  }

  Stream<bool> get batterySavingEveStm {
    return _bleEveChannels.batterySavingEveStm;
  }

  Future<void> sendBatterySaving(bool enable) async {
    await mConnMethodChannel
        .invokeMethod("sendBatterySaving", <String, bool>{"enable": enable});
  }

  Future<void> get queryBatterySaving async {
    await mConnMethodChannel.invokeMethod("queryBatterySaving");
  }

  Future<PillReminderCallback> get queryPillReminder async {
    String jsonStr = await mConnMethodChannel.invokeMethod("queryPillReminder");
    PillReminderCallback pillReminderCallback =
        pillReminderCallbackFromJson(jsonStr);
    return pillReminderCallback;
  }

  Future<void> sendPillReminder(PillReminderBean info) async {
    String jsonStr = pillReminderBeanToJson(info);
    await mConnMethodChannel.invokeMethod(
        "sendPillReminder", <String, String>{"pillReminderInfo": jsonStr});
  }

  Future<void> deletePillReminder(int id) async {
    await mConnMethodChannel
        .invokeMethod("deletePillReminder", <String, int>{"id": id});
  }

  Future<void> get clearPillReminder async {
    await mConnMethodChannel.invokeMethod("clearPillReminder");
  }

  Future<bool> get queryWakeState async {
    bool state = await mConnMethodChannel.invokeMethod("queryWakeState");
    return state;
  }

  Future<void> sendWakeState(bool enable) async {
    await mConnMethodChannel
        .invokeMethod("sendWakeState", <String, bool>{"enable": enable});
  }

  Stream<TrainBean> get trainingEveStm {
    return _bleEveChannels.trainingEveStm;
  }

  Future<void> get queryHistoryTraining async {
    await mConnMethodChannel.invokeMethod("queryHistoryTraining");
  }

  Future<void> queryTraining(int id) async {
    await mConnMethodChannel
        .invokeMethod("queryTraining", <String, int>{"id": id});
  }

  Stream<dynamic> get sosChangeEveStm {
    return _bleEveChannels.sosChangeEveStm;
  }

  Future<int> createBond(List<int> keys) async {
    String keysStr = keys.join();
    return await mConnMethodChannel
        .invokeMethod("createBond", <String, String>{'keys': keysStr});
  }

  Stream<HrvHandlerBean> get newHrvEveStm {
    return _bleEveChannels.newHrvEveStm;
  }

  Future<void> get querySupportNewHrv async {
    await mConnMethodChannel.invokeMethod("querySupportNewHrv");
  }

  Future<void> get startMeasureNewHrv async {
    await mConnMethodChannel.invokeMethod("startMeasureNewHrv");
  }

  Future<void> get stopMeasureNewHrv async {
    await mConnMethodChannel.invokeMethod("stopMeasureNewHrv");
  }

  Future<void> get queryHistoryNewHrv async {
    await mConnMethodChannel.invokeMethod("queryHistoryNewHrv");
  }

  Stream<StressHandlerBean> get stressEveStm {
    return _bleEveChannels.stressEveStm;
  }

  Future<void> get querySupportStress async {
    await mConnMethodChannel.invokeMethod("querySupportStress");
  }

  Future<void> get startMeasureStress async {
    await mConnMethodChannel.invokeMethod("startMeasureStress");
  }

  Future<void> get stopMeasureStress async {
    await mConnMethodChannel.invokeMethod("stopMeasureStress");
  }

  Future<void> get queryHistoryStress async {
    await mConnMethodChannel.invokeMethod("queryHistoryStress");
  }

  Future<void> get enableTimingStress async {
    await mConnMethodChannel.invokeMethod("enableTimingStress");
  }

  Future<void> get disableTimingStress async {
    await mConnMethodChannel.invokeMethod("disableTimingStress");
  }

  Future<void> get queryTimingStressState async {
    await mConnMethodChannel.invokeMethod("queryTimingStressState");
  }

  Future<void> queryTimingStress(String stressDate) async {
    await mConnMethodChannel.invokeMethod(
        "queryTimingStress", <String, String>{"stressDate": stressDate});
  }

  Future<ElectronicCardCountInfoBean> get queryElectronicCardCount async {
    String jsonStr = await mConnMethodChannel.invokeMethod("queryElectronicCardCount");
    ElectronicCardCountInfoBean electronicCardCountInfo = electronicCardCountInfoBeanFromJson(jsonStr);
    return electronicCardCountInfo;
  }

  Future<void> sendElectronicCard(
      ElectronicCardInfoBean electronicCardInfo) async {
    String jsonStr = electronicCardInfoBeanToJson(electronicCardInfo);
    await mConnMethodChannel.invokeMethod(
        "sendElectronicCard", <String, String>{"electronicCardInfo": jsonStr});
  }

  Future<void> deleteElectronicCard(int id) async {
    await mConnMethodChannel
        .invokeMethod("deleteElectronicCard", <String, int>{"id": id});
  }

  Future<ElectronicCardInfoBean> queryElectronicCard(int id) async {
    String jsonStr = await mConnMethodChannel
        .invokeMethod("queryElectronicCard", <String, int>{"id": id});
    ElectronicCardInfoBean electronicCardInfo = electronicCardInfoBeanFromJson(jsonStr);
    return electronicCardInfo;
  }

  Future<void> sendElectronicCardList(List idList) async {
    String idListStr = jsonEncode(idList);
    await mConnMethodChannel.invokeMethod(
        "sendElectronicCardList", <String, String>{"idList": idListStr});
  }

  Stream<CalendarEventBean> get calendarEventEveStem {
    return _bleEveChannels.calendarEventEveStem;
  }

  Future<void> get querySupportCalendarEvent async {
    await mConnMethodChannel.invokeMethod("querySupportCalendarEvent");
  }

  Future<void> sendCalendarEvent(
      CalendarEventInfoBean calenderEventInfo) async {
    String jsonStr = calendarEventInfoBeanToJson(calenderEventInfo);
    await mConnMethodChannel.invokeMethod(
        "sendCalendarEvent", <String, String>{"calenderEventInfo": jsonStr});
  }

  Future<void> deleteCalendarEvent(int id) async {
    await mConnMethodChannel
        .invokeMethod("deleteCalendarEvent", <String, int>{"id": id});
  }

  Future<void> queryCalendarEvent(int id) async {
    await mConnMethodChannel
        .invokeMethod("queryCalendarEvent", <String, int>{"id": id});
  }

  Future<void> sendCalendarEventReminderTime(
      CalendarEventReminderTimeBean calendarEventReminderTime) async {
    String jsonStr =
        calendarEventReminderTimeBeanToJson(calendarEventReminderTime);
    await mConnMethodChannel.invokeMethod("sendCalendarEventReminderTime",
        <String, String>{"calendarEventReminderTime": jsonStr});
  }

  Future<void> get queryCalendarEventReminderTime async {
    await mConnMethodChannel.invokeMethod("queryCalendarEventReminderTime");
  }

  Future<void> get clearCalendarEvent async {
    await mConnMethodChannel.invokeMethod("clearCalendarEvent");
  }

  Stream<int> get gpsChangeEveStm {
    return _bleEveChannels.gpsChangeEveStm;
  }

  Future<void> get queryHistoryGps async {
    await mConnMethodChannel.invokeMethod("queryHistoryGps");
  }

  Future<void> queryGpsDetail(int seconds) async {
    await mConnMethodChannel.invokeMethod("queryHistoryGps", <String, int>{"seconds": seconds});
  }

  Future<void> sendVibrationStrength(int value) async {
    await mConnMethodChannel.invokeMethod("sendVibrationStrength", <String, int>{"value": value});
  }

  Future<VibrationStrength> get queryVibrationStrength async {
    String jsonStr = await mConnMethodChannel.invokeMethod("queryVibrationStrength");
    return vibrationStrengthFromJson(jsonStr);
  }

  Future<bool> uploadLocalFile(String filePath) async {
    File file = File(filePath);
    if (await file.exists()) {
      CustomizeWatchFaceBean bean = CustomizeWatchFaceBean(file: filePath, index: 5);
      /// 调用表盘上传接口
      try {
        // print("开始上传");
        await sendWatchFace(SendWatchFaceBean(watchFaceFlutterBean: bean, timeout: 30));
        return true;
      } catch (e) {
        // print(e);
        // print("上传超时");
        return false;
      }
    }else{
      // print("没有这个文件");
      return false;
    }
  }
}
